@@ -1,4 +1,4 @@
-// $Id: header_frame.cpp,v 1.11 2000/09/11 07:46:32 eldamitri Exp $
+// $Id: header_frame.cpp,v 1.22 2002/07/02 22:13:10 t1mpy Exp $
 
 // id3lib: a C++ library for creating and manipulating id3v1/v2 tags
 // Copyright 1999, 2000  Scott Thomas Haug
@@ -24,15 +24,16 @@
 // id3lib.  These files are distributed with id3lib at
 // http://download.sourceforge.net/id3lib/
 
-#include <string.h>
+
 #include <memory.h>
 #include "header_frame.h"
-#include "error.h"
-#include "utils.h"
+#include "id3/utils.h" // has <config.h> "id3/id3lib_streams.h" "id3/globals.h" "id3/id3lib_strings.h"
+#include "frame_def.h"
+#include "field_def.h"
+#include "field_impl.h"
+#include "io_helpers.h"
 
-#if defined HAVE_CONFIG_H
-#include <config.h>
-#endif
+using namespace dami;
 
 void ID3_FrameHeader::SetUnknownFrame(const char* id)
 {
@@ -87,83 +88,93 @@ size_t ID3_FrameHeader::Size() const
     _info->frame_bytes_flags;
 }
 
-size_t ID3_FrameHeader::Parse(const uchar *buffer, size_t size)
+bool ID3_FrameHeader::Parse(ID3_Reader& reader)
 {
-  size_t index = 0;
-  char text_id[5];
-
+  ID3D_NOTICE( "ID3_FrameHeader::Parse(): getCur() = " << reader.getCur() );
+  io::ExitTrigger et(reader);
   if (!_info)
   {
-    return 0;
+    return false;
+  }
+  if (reader.getEnd() < reader.getCur() + 10)
+  {
+    return false;
   }
 
-  strncpy(text_id, (char *) buffer, _info->frame_bytes_id);
-  text_id[_info->frame_bytes_id] = '\0';
-  index += _info->frame_bytes_id;
+  String textID = io::readText(reader, _info->frame_bytes_id);
 
-  ID3_FrameID fid = ID3_FindFrameID(text_id);
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: textID = " << textID );
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: getCur() = " << reader.getCur() );
+
+  ID3_FrameID fid = ID3_FindFrameID(textID.c_str());
   if (ID3FID_NOFRAME == fid)
   {
-    this->SetUnknownFrame(text_id);
+    this->SetUnknownFrame(textID.c_str());
+    ID3D_NOTICE( "ID3_FrameHeader::Parse: unknown frame id" );
   }
   else
   {
     this->SetFrameID(fid);
   }
 
-  this->SetDataSize(ParseNumber(&buffer[index], _info->frame_bytes_size));
-  index += _info->frame_bytes_size;
+  uint32 dataSize = io::readBENumber(reader, _info->frame_bytes_size);
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: dataSize = " << dataSize );
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: getCur() = " << reader.getCur() );
+  this->SetDataSize(dataSize);
 
-  _flags.add(ParseNumber(&buffer[index], _info->frame_bytes_flags));
-  index += _info->frame_bytes_flags;
-  
-  return index;
+  uint32 flags = io::readBENumber(reader, _info->frame_bytes_flags);
+  _flags.add(flags);
+
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: flags = " << flags );
+  ID3D_NOTICE( "ID3_FrameHeader::Parse: getCur() = " << reader.getCur() );
+  et.setExitPos(reader.getCur());
+
+  return true;
 }
 
-size_t ID3_FrameHeader::Render(uchar *buffer) const
+void ID3_FrameHeader::Render(ID3_Writer& writer) const
 {
   size_t size = 0;
 
   if (NULL == _frame_def)
   {
     // TODO: log this
-    return 0;
+    ID3D_WARNING( "ID3_FrameHeader::Render(): _frame_def is NULL!" );
+    return;
     //ID3_THROW(ID3E_InvalidFrameID);
   }
-  char *text_id;
+  char *textID;
   if (_info->frame_bytes_id == strlen(_frame_def->sShortTextID))
   {
-    text_id = _frame_def->sShortTextID;
+    textID = _frame_def->sShortTextID;
   }
   else
   {
-    text_id = _frame_def->sLongTextID;
+    textID = _frame_def->sLongTextID;
   }
 
-  memcpy(&buffer[size], (uchar *) text_id, _info->frame_bytes_id);
-  size += _info->frame_bytes_id;
-  
-  size += RenderNumber(&buffer[size], _data_size, _info->frame_bytes_size);
-  size += RenderNumber(&buffer[size], _flags.get(), _info->frame_bytes_flags);
-  
-  return size;
+  ID3D_NOTICE( "ID3_FrameHeader::Render(): writing " << textID << ", " << (int) _info->frame_bytes_size << " bytes");
+  writer.writeChars((uchar *) textID, _info->frame_bytes_id);
+
+  io::writeBENumber(writer, _data_size, _info->frame_bytes_size);
+  io::writeBENumber(writer, _flags.get(), _info->frame_bytes_flags);
 }
 
 const char* ID3_FrameHeader::GetTextID() const
 {
-  char *text_id = "";
+  char *textID = "";
   if (_info && _frame_def)
   {
     if (_info->frame_bytes_id == strlen(_frame_def->sShortTextID))
     {
-      text_id = _frame_def->sShortTextID;
+      textID = _frame_def->sShortTextID;
     }
     else
     {
-      text_id = _frame_def->sLongTextID;
+      textID = _frame_def->sLongTextID;
     }
   }
-  return text_id;
+  return textID;
 }
 
 ID3_FrameHeader& ID3_FrameHeader::operator=(const ID3_FrameHeader& hdr)
@@ -227,3 +238,4 @@ bool ID3_FrameHeader::Clear()
   }
   return changed;
 }
+
